@@ -18,304 +18,13 @@
 #
 ###############################################################################
 
-#' @importFrom SLOPE prox_sorted_L1
-#' @importFrom stats lm pchisq qchisq qnorm rnorm sd uniroot coef predict
+#' @importFrom stats lm pchisq qchisq qnorm sd uniroot coef predict
 #' @importFrom utils head
+#' @importFrom Rcpp sourceCpp
+#' @useDynLib grpSLOPE, .registration = TRUE
 NULL
 #> NULL
 
-#' Prox for group SLOPE
-#'
-#' Evaluate the proximal mapping for the group SLOPE problem.
-#'
-#' \code{proxGroupSortedL1} evaluates the proximal mapping of the group SLOPE problem
-#' by reducing it to the prox for the (regular) SLOPE and then applying the fast prox
-#' algorithm for the Sorted L1 norm.
-#'
-#' @param y The response vector
-#' @param group Either a vector or an object of class \code{groupID} (e.g. as produced by 
-#'   \code{\link{getGroupID}}), which is describing the grouping structure. If it is
-#'   a vector, then it should contain a group id for each predictor variable.
-#' @param lambda A decreasing sequence of regularization parameters \eqn{\lambda}
-#' @param ... Options passed to \code{\link[SLOPE]{prox_sorted_L1}}
-#'
-#' @examples
-#' grp <- c(0,0,0,1,1,0,2,1,0,2)
-#' proxGroupSortedL1(y = 1:10, group = grp, lambda = c(10, 9, 8))
-#' #  [1] 0.2032270 0.4064540 0.6096810 0.8771198 1.0963997 1.2193620 1.3338960
-#' #  [8] 1.7542395 1.8290430 1.9055657
-#'
-#' @references M. Bogdan, E. van den Berg, C. Sabatti, W. Su, E. Candes (2015), \emph{SLOPE -- Adaptive variable selection via convex optimization}, \url{http://arxiv.org/abs/1407.3824}
-#'
-#' @export
-proxGroupSortedL1 <- function(y, group, lambda, ...) {
-  if (inherits(group, "groupID")) {
-    n.group <- length(group)
-    group.id <- group
-  } else {
-    n.group <- length(unique(group))
-    group.id <- getGroupID(group)
-  }
-
-  if (length(lambda) != n.group) {
-    stop("Length of lambda should be equal to the number of groups.")
-  }
-
-  # compute Euclidean norms for groups in y
-  group.norm <- rep(NA, n.group)
-  for (i in 1:n.group){
-    selected <- group.id[[i]]
-    group.norm[i] <- norm(as.matrix(y[selected]), "f")
-  }
-
-  # get Euclidean norms of the solution vector
-  prox.norm <- SLOPE::prox_sorted_L1(group.norm, lambda, ...)
-
-  # compute the solution
-  prox.solution <- rep(NA, length(y))
-  for (i in 1:n.group){
-    selected <- group.id[[i]]
-    prox.solution[selected] <- prox.norm[i] / group.norm[i] * y[selected]
-  }
-
-  return(prox.solution)
-}
-
-#' Proximal gradient method for Group SLOPE
-#'
-#' Compute the coefficient estimates for the Group SLOPE problem.
-#'
-#' \code{proximalGradientSolverGroupSLOPE} computes the coefficient estimates
-#' for the Group SLOPE model. The employed optimization algorithm is FISTA with 
-#' backtracking Lipschitz search.
-#'
-#' @param y the response vector
-#' @param A the model matrix
-#' @param group A vector describing the grouping structure. It should 
-#'   contain a group id for each predictor variable.
-#' @param wt A vector of weights (per coefficient)
-#' @param lambda A decreasing sequence of regularization parameters \eqn{\lambda}
-#' @param max.iter Maximal number of iterations to carry out
-#' @param verbose A \code{logical} specifying whether to print output or not
-#' @param dual.gap.tol The tolerance used in the stopping criteria for the duality gap
-#' @param infeas.tol The tolerance used in the stopping criteria for the infeasibility
-#' @param x.init An optional initial value for the iterative algorithm
-#' @param ... Options passed to \code{\link[SLOPE]{prox_sorted_L1}}
-#'
-#' @return A list with members:
-#'   \describe{
-#'     \item{x}{Solution (n-by-1 matrix)}
-#'     \item{status}{Convergence status: 1 if optimal, 2 if iteration limit reached}
-#'     \item{L}{Approximation of the Lipschitz constant (step size)}
-#'     \item{iter}{Iterations of the proximal gradient method}
-#'     \item{L.iter}{Total number of iterations spent in Lischitz search}
-#'   }
-#'
-#' @examples
-#' set.seed(1)
-#' A   <- matrix(runif(100, 0, 1), 10, 10)
-#' grp <- c(0, 0, 1, 1, 2, 2, 2, 2, 2, 3)
-#' wt  <- c(2, 2, 2, 2, 5, 5, 5, 5, 5, 1)
-#' x   <- c(0, 0, 5, 1, 0, 0, 0, 1, 0, 3)
-#' y   <- A %*% x
-#' lam <- 0.1 * (10:7)
-#' result <- proximalGradientSolverGroupSLOPE(y=y, A=A, group=grp, wt=wt, lambda=lam, verbose=FALSE)
-#' result$x
-#' #           [,1]
-#' #  [1,] 0.000000
-#' #  [2,] 0.000000
-#' #  [3,] 3.856005
-#' #  [4,] 2.080736
-#' #  [5,] 0.000000
-#' #  [6,] 0.000000
-#' #  [7,] 0.000000
-#' #  [8,] 0.000000
-#' #  [9,] 0.000000
-#' # [10,] 3.512833
-#'
-#' @references D. Brzyski, A. Gossmann, W. Su, and M. Bogdan (2016) \emph{Group SLOPE -- adaptive selection of groups of predictors}, \url{https://arxiv.org/abs/1610.04960}
-#' @references A. Gossmann, S. Cao, Y.-P. Wang (2015) \emph{Identification of Significant Genetic Variants via SLOPE, and Its Extension to Group SLOPE}, \url{http://dx.doi.org/10.1145/2808719.2808743}
-#'
-#' @export
-proximalGradientSolverGroupSLOPE <- function(y, A, group, wt, lambda, max.iter=1e4,
-                                             verbose=FALSE, dual.gap.tol=1e-6, 
-                                             infeas.tol=1e-6, x.init=vector(), ...)
-{
-  # This function is based on the source code available from
-  # http://statweb.stanford.edu/~candes/SortedL1/software.html
-  # under the GNU GPL-3 licence.
-  #
-  # Original implementation: Copyright 2013, M. Bogdan, E. van den Berg, W. Su, and E.J. Candes
-  # Modifications: Copyright 2015, Alexej Gossmann
-  
-  # Initialize ---------------------------------------------------------------
-
-  # Prepare grouping information
-  group.id <- getGroupID(group)
-  n.group  <- length(group.id)
-  
-  # Ensure that lambda is non-increasing and of right length
-  n.lambda <- length(lambda)
-  if ((n.lambda > 1) && any(lambda[2:n.lambda] > lambda[1:n.lambda-1])) {
-    stop("Lambda must be non-increasing.")
-  }
-  if (n.lambda != n.group) {
-    stop("Lambda must have exactly as many entries as there are groups.")
-  }
-
-  # Adjust matrix for prior weights
-  Dinv <- diag(1/wt)
-  A    <- A %*% Dinv
-  p    <- ncol(A)
-
-  # Auxilliary function to record output information
-  recordResult <- function(b, status, L, iter, L.iter) {
-    result           <- list()
-    result$x         <- Dinv %*% b
-    result$status    <- status
-    result$L         <- L
-    result$iter      <- iter
-    result$L.iter    <- L.iter
-    return(result)
-  }
-
-  # Run regular SLOPE if all groups are singletons
-  if (length(group.id) == p) {
-    if (length(x.init) == 0) x.init <- matrix(0,p,1)
-    sol <- SLOPE::SLOPE_solver(A=A, b=y, lambda=lambda, initial=x.init,
-                               max_iter=max.iter, tol_infeas=infeas.tol,
-                               tol_rel_gap=dual.gap.tol)
-    status <- 2
-    if (sol$optimal) status <- 1
-    result <- recordResult(b=matrix(sol$x, c(p,1)), status=status, L=sol$lipschitz,
-                           iter=sol$iter, L.iter=NULL)
-    return(result)
-  }
-  
-  # Get initial lower bound on the Lipschitz constant
-  rand.mat <- matrix(rnorm(p),c(p,1))
-  rand.mat <- rand.mat / norm(rand.mat, "f")
-  rand.mat <- t(A) %*% (A %*% rand.mat)
-  L <- norm(rand.mat, "f")
-  
-  # Set constants
-  STATUS_RUNNING    <- 0
-  STATUS_OPTIMAL    <- 1
-  STATUS_ITERATIONS <- 2
-  STATUS_MSG        <- c('Optimal','Iteration limit reached')
-  
-  # Initialize parameters and iterates
-  if (length(x.init) == 0) x.init <- matrix(0,p,1)
-  tt       <- 1
-  eta      <- 2
-  lambda   <- as.matrix(lambda)
-  y        <- as.matrix(y)
-  x        <- x.init
-  b        <- x
-  Ax       <- A %*% x
-  f        <- Inf
-  f.prev   <- Inf
-  iter     <- 0
-  L.iter   <- 0
-  status   <- STATUS_RUNNING
-  duality.gap <- Inf
-  infeasibility <- Inf
-  
-  if (verbose == TRUE) {
-    printf <- function(...) invisible(cat(sprintf(...)))
-    printf('%5s  %9s  %9s  %9s\n', 'Iter', '||r||_2', 'Dual. gap', 'Infeas.')
-  }
-  
-  # Main loop ---------------------------------------------------------
-  while (TRUE)
-  {    
-    # Compute the gradient
-    r <- (A %*% b) - y
-    g <- crossprod(A, r)
-    
-    # Stopping criteria --------------------------------------------
-
-    # Compute duality gap (eq. (B.20) in Appendix B in Brzyski et. al. (2016))
-    b.norms <- rep(NA, n.group)
-    for (i in 1:n.group) {
-      selected <- group.id[[i]]
-      b.norms[i] <- norm(as.matrix(b[selected]), "f")
-    }
-    b.norms.sorted <- sort(b.norms, decreasing=TRUE)
-    duality.gap <- crossprod(b, g) + crossprod(lambda, b.norms.sorted)
-
-    # Compute the infeasibility
-    # (a comment on the derivation of this infeasibility criterion can be found at:
-    # https://github.com/agisga/grpSLOPE/blob/gh-pages/_posts/outdated_2015-12-4-Infeasibility.md)
-    g.norms <- rep(NA, n.group)
-    for (i in 1:n.group) {
-      selected <- group.id[[i]]
-      g.norms[i] <- norm(as.matrix(g[selected]), "f")
-    }
-    g.norms.sorted  <- sort(g.norms, decreasing=TRUE)
-    infeasibility <- max(max(cumsum(g.norms.sorted-lambda)),0)
-
-    # Format string
-    if (verbose == TRUE) str <- sprintf('   %9.2e  %9.2e', duality.gap, infeasibility)
-     
-    # Check stopping criteria
-    if ((duality.gap < dual.gap.tol) && (infeasibility < infeas.tol)) {
-      status <- STATUS_OPTIMAL
-    }
-    
-    if (verbose == TRUE) {
-      printf('%5d  %9.2e%s\n', iter, f, str)
-    }
-
-    if ((status == 0) && (iter >= max.iter)) {
-      status <- STATUS_ITERATIONS
-    }
-
-    if (status != 0) {
-      if (verbose == TRUE) {
-        printf('Exiting with status %d -- %s\n', status, STATUS_MSG[[status]])
-      }
-      break
-    }
-    # (stopping criteria ends) ------------------
-    
-    # Increment iteration count
-    iter <- iter + 1
-    
-    # Keep copies of previous values
-    f.prev  <- as.double(crossprod(r)) / 2
-    Ax.prev <- Ax
-    x.prev  <- x
-    b.prev  <- b
-    tt.prev <- tt
-    
-    # Lipschitz search
-    while (TRUE) {
-      x  <- proxGroupSortedL1(y = b - (1/L)*g, group = group.id, lambda = lambda/L, ...)
-      d  <- x - b
-      Ax <- A %*% x
-      r  <- Ax-y
-      f  <- as.double(crossprod(r))/2
-      qq <- f.prev + as.double(crossprod(d,g)) + (L/2)*as.double(crossprod(d))
-                              
-      L.iter <- L.iter + 1
-                              
-      if (qq >= f*(1-1e-12))
-        break
-      else
-        L <- L * eta
-    }
-    
-    # Update
-    tt <- (1 + sqrt(1 + 4*tt^2)) / 2
-    b  <- x + ((tt.prev - 1) / tt) * (x - x.prev)
-  } # while (TRUE)
-  
-  
-  result <- recordResult(b=b, status=status, L=L, iter=iter, L.iter=L.iter)
-  return(result)
-}
-
 
 #' Regularizing sequence for Group SLOPE
 #'
@@ -363,6 +72,7 @@ proximalGradientSolverGroupSLOPE <- function(y, A, group, wt, lambda, max.iter=1
 #' # lambda.corrected 1.880540 1.729811 1.637290 1.568971 1.514028 1.467551
 #'
 #' @references D. Brzyski, A. Gossmann, W. Su, and M. Bogdan (2016) \emph{Group SLOPE -- adaptive selection of groups of predictors}, \url{https://arxiv.org/abs/1610.04960}
+#' @references D. Brzyski, A. Gossmann, W. Su, and M. Bogdan (2019) \emph{Group SLOPE -- adaptive selection of groups of predictors}. Journal of the American Statistical Association 114 (525): 419–33.
 #' @export
 lambdaGroupSLOPE <- function(method, fdr, group, wt, n.obs=NULL)
 {
@@ -446,7 +156,7 @@ lambdaGroupSLOPE <- function(method, fdr, group, wt, n.obs=NULL)
 #' @param dual.gap.tol See \code{\link{proximalGradientSolverGroupSLOPE}}.
 #' @param infeas.tol See \code{\link{proximalGradientSolverGroupSLOPE}}.
 #' @param x.init See \code{\link{proximalGradientSolverGroupSLOPE}}.
-#' @param ... Options passed to \code{\link[SLOPE]{prox_sorted_L1}}
+#' @param ... Options passed to \code{\link{prox_sorted_L1}}
 #'
 #' @return A list with members:
 #'   \describe{
@@ -484,12 +194,13 @@ lambdaGroupSLOPE <- function(method, fdr, group, wt, n.obs=NULL)
 #' #  2.905449  5.516103  8.964201 10.253792  0.000000  0.000000 
 #'
 #' @references D. Brzyski, A. Gossmann, W. Su, and M. Bogdan (2016) \emph{Group SLOPE -- adaptive selection of groups of predictors}, \url{https://arxiv.org/abs/1610.04960}
+#' @references D. Brzyski, A. Gossmann, W. Su, and M. Bogdan (2019) \emph{Group SLOPE -- adaptive selection of groups of predictors}. Journal of the American Statistical Association 114 (525): 419–33.
 #'
 #' @export
 grpSLOPE <- function(X, y, group, fdr, lambda = "corrected", sigma = NULL,
                      verbose = FALSE, orthogonalize = NULL, normalize = TRUE,
                      max.iter=1e4, dual.gap.tol=1e-6, infeas.tol=1e-6,
-                     x.init=vector(), ...) {
+                     x.init=NULL, ...) {
 
   # check inputs for NA or NaN
   if (anyNA(X)) { stop("Some entries of X are NA or NaN!") }
@@ -514,7 +225,7 @@ grpSLOPE <- function(X, y, group, fdr, lambda = "corrected", sigma = NULL,
       stop("X cannot be normalized (probably some column has sample variance equal to 0).")
     }
     X <- X %*% diag(X.scaling)
-    # (3) center y to have unit norm
+    # (3) center y
     y.mean <- mean(y)
     y <- y - y.mean
   }
